; =======================================================================================
; Title .........: Check Form Generator
; Description ...: Check Form Generator
; Requirement(s).: compatible Autoit v3.3.14.2
; Author(s) .....: Giovanni Rossati (El Condor)
; Version........: 0.2.0
; Date...........: 12 December 2017
; =======================================================================================
#include-once
#include <Array.au3>
Func fhCheck($records)
   $controlsDict = ObjCreate("Scripting.Dictionary")   ; Create a Scripting Dictionary of widgets
   $pseudoDict = ObjCreate("Scripting.Dictionary")
   Global $fhaRecords = StringRegExp($records,"([^;]+);?",3) ; array splitted by ; without empty
   $error = ""
   For $iControls=0 To UBound($fhaRecords)-1		; control size and parameters arrangement (Type upper, Label missing, split key value, deMask)
	  $aEl = StringSplit($fhaRecords[$iControls] & ",,,,,,,",",",2)
	  $aEl[0] = StringUpper($aEl[0])
	  If StringLeft($fhaRecords[$iControls],2) = "//" Then ContinueLoop		; comment
	  Switch $aEl[0]

		Case "S","SLIDER"
			  $aLim = StringRegExp($aEl[5],'[-+]?[0-9]+\.?[0-9]*', 3)
			   If UBound($aLim) <> 2 Then
				  $error = $error & "Warning: slider must have two numerics values" & @CRLF
			   Else
				  If Number($aLim[0]) > Number($aLim[1]) Then _ArraySwap($aLim,0, 1)
				  If $aEl[4] <> "" And (Number($aEl[4]) < Number($aLim[0]) Or Number($aEl[4]) > Number($aLim[1])) Then
					 $error = $error & "Error Slider default value out of Range" & @CRLF
				  Else
					 $aLim = StringRegExp($aEl[3],'[0-9]+', 3)
					 If UBound($aLim) = 0 Then $error = $error & "Error type " & $aEl[0] & " Lack of length" & @CRLF
				  EndIf
			   EndIf
		 Case "T","P","F","D","S","N","DATE","TIME","TEXT","PSW","FILE"
			$aLim = StringRegExp($aEl[3],'[0-9]+', 3)
			If UBound($aLim) = 0 Then $error = $error & "Error type " & $aEl[0] & " Lack of length" & @CRLF
			If $aEl[0] = "T" Or $aEl[0] = "TEXT" Then
			   If StringRegExp($aEl[5],'^[Ss]?\d+\.?\d*$|^\s*$',$STR_REGEXPMATCH) = 0 Then $error &= "Error: invalid numeric type on " & $aEl[1]  & @CRLF
			EndIf
		 Case "BL"
		 Case "B","TB"
		 Case "X"
			If $aEl[5] = "" Then $error = $error & "Error: lack of Call Back function" & @CRLF
		 Case "CKB"
			   If $aEl[4] <> "" AND $aEl[4] <> "1" Then $error = $error & "Error; lack of Call Back function" & @CRLF
		 Case "REQUIRED","AFTER","CONTROL","CHECK","TIMER","MENU","SUBMENU"
			$pseudoDict.Add($aEl[0] & $aEl[1], $aEl)
		 Case "CMB","CMT","R"
			   If $aEl[5] = "" Then $error = $error & "Warning: Extra field of " & $aEl[0] & " must be: [key=]value[|[key=]value ...]" & @CRLF
		Case "C","COMMENT","U"
		Case Else
			$error = $error & "Error: " & $aEl[0] & " unknown Type" & @CRLF
	  EndSwitch
	  Switch $aEl[0]
		 Case "REQUIRED","AFTER","CONTROL","CHECK","TIMER","MENU","SUBMENU"
		 Case Else
			$controlsDict.Item($aEl[1]) = $aEl
	  EndSwitch
   Next
;	control of pseudotypes
   For $pseudo in $pseudoDict		; key = name, value = ID
	   ConsoleWrite('@@ Debug(' & @ScriptLineNumber & ') : $pseudo = ' & $pseudo & @CRLF & '>Error code: ' & @error & @CRLF) ;### Debug Console
	  $aEl =  $pseudoDict.Item($pseudo)
	  If StringLeft($pseudo,5) = "AFTER" Then
		 If Not $controlsDict.Exists($aEl[1]) Then  $error &= "Error: AFTER control " & $aEl[1]  & " unknown" & @CRLF
		 If Not $controlsDict.Exists($aEl[2]) Then  $error &= "Error: AFTER control " & $aEl[2]  & " unknown" & @CRLF
 	  ElseIf StringLeft($pseudo,5)  = "CHECK" Or StringLeft($pseudo,7) = "CONTROL" Then
		 If Not $controlsDict.Exists($aEl[1]) Then  $error &= "Error: CHECK control " & $aEl[1]  & " unknown" & @CRLF
 	  ElseIf StringLeft($pseudo,4)  = "MENU" Then
;		 If Not $controlsDict.Exists($aEl[1]) Then  $error &= "Error: CHECK control " & $aEl[1]  & " unknown" & @CRLF
	  ElseIf StringLeft($pseudo,7)  = "SUBMENU" Then
;	 	If Not $controlsDict.Exists($aEl[2]) Then  $error &= "Error: SUBMENU " & $aEL[1] & ": parent " & $aEl[2]  & " unknown" & @CRLF
 	  ElseIf StringLeft($pseudo,5)  = "TIMER" Then
;		 If Not $controlsDict.Exists($aEl[2]) Then  $error &= "Error: SubMenu " & $aEl[1]  & " unknown" & @CRLF
 	  ElseIf StringLeft($pseudo,8) = "REQUIRED" Then
		 For $i = 1 To UBound($aEl) - 1
			If $aEl[$i] <> "" And Not $controlsDict.Exists($aEl[$i]) Then  $error &= "Error: REQUIRED control " & $aEl[$i]  & " unknown" & @CRLF
		 Next
	  EndIf
   Next
   return ($error = "") ? "No errors!": $error
EndFunc
Func trace($ID)
   If $ID > 0 Then ConsoleWrite($ID & " " & @CRLF)
EndFunc